import React, { FC, useState } from 'react';
import { View } from 'react-native';
import { CalendarControls } from './components/calendarControls';
import { CalendarMonthDates } from './components/calendarMonthDates';
import { CalendarWeekDays } from './components/calendarWeekDays';
import { getDatesOfMonthData,
  getSelectedDate,
  getNameOfDaysOfTheWeekData } from './utils';
import { getCurrentDate, getNameOfMonth } from '../../../utils';
import { CalendarProps, 
  DateNumberMonthNumberAndYear, 
  HandleOnSelectedDate, 
  HandleOnGoToDate, 
  MonthNumber } from './calendarModel';
import { styles } from './styles';

type GetSelectedYear = (currentYear: number, defaultYear?: number) => number;
const getSelectedYear: GetSelectedYear = (currentYear, defaultYear) => {
  return defaultYear || currentYear;
};

type GetSelectedMonthNumber = (currentMonth: MonthNumber, defaultMonth?: MonthNumber) => MonthNumber;
const getSelectedMonthNumber: GetSelectedMonthNumber = (currentMonth, defaultMonth) => {
  return defaultMonth || currentMonth;
};

type GetMonthName = (defaultMonthNumber?: MonthNumber) => string;
const getMonthName: GetMonthName = (defaultMonthNumber) => {
  return defaultMonthNumber
    ? getNameOfMonth(defaultMonthNumber)
    : getNameOfMonth();
};

export const Calendar: FC<CalendarProps> = ({
  defaultMonthNumberAndYear: defaultMonthAndYear,
  onMonthNumberAndYearNavigate: onDateNavigate,
  onDateClick,
  ...categoriesParams
}) => {
  const currentDate: DateNumberMonthNumberAndYear = getCurrentDate();
  const selectedMonthName: string = getMonthName(defaultMonthAndYear?.monthNumber);
  const selectedMonthNumber: MonthNumber = getSelectedMonthNumber(
    currentDate.monthNumber,
    defaultMonthAndYear?.monthNumber,
  );
  const selectedYear: number = getSelectedYear(currentDate.year, defaultMonthAndYear?.year);
  const nameOfWeekDaysData = getNameOfDaysOfTheWeekData();

  const [selectedDate, setSelectedDate] = useState<DateNumberMonthNumberAndYear | null>(null);
  const [monthName, setMonthName] = useState<string>(selectedMonthName);
  const [monthNumber, setMonthNumber] = useState<MonthNumber>(selectedMonthNumber);
  const [year, setYear] = useState<number>(selectedYear);

  const datesListOfSelectedMonthData = getDatesOfMonthData({
    currentDate,
    monthNumber,
    year: year,
    selectedDate,
    ...categoriesParams,
  });

  const handleOnGoToDate: HandleOnGoToDate = (params) => {
    const { monthNumber, year } = getSelectedDate(params);
    const monthName = getMonthName(monthNumber);

    setMonthNumber(monthNumber);
    setYear(year);
    setMonthName(monthName);

    onDateNavigate?.(monthNumber, year);
  };

  const handleOnSelectedDate: HandleOnSelectedDate = (dateNumber) => {
    setSelectedDate({
      dateNumber,
      monthNumber,
      year,
    });

    onDateClick?.({
      dateNumber,
      monthNumber,
      year,
    });
  };

  return (
    <View style={styles.calendarWrapper}>
      <CalendarControls
        monthName={monthName}
        monthNumber={monthNumber}
        yearData={year}
        onGoToMonthNumberAndYear={handleOnGoToDate}
      />
      <View style={styles.calendarGap} />
      <CalendarWeekDays dayOfTheWeekData={nameOfWeekDaysData} />
      <View style={styles.calendarGap} />
      <CalendarMonthDates
        datesData={datesListOfSelectedMonthData}
        onDatePress={handleOnSelectedDate}
      />
    </View>
  );
};
